@@ -27,7 +27,7 @@ typedef short            int16_t;
 typedef unsigned int     uint32_t;
 typedef int              int32_t;
 #if defined(_MSC_VER)
-typedef char             int8_t;
+typedef signed char      int8_t;
 typedef unsigned __int64 uint64_t;
 typedef          __int64 int64_t;
 #else
