@@ -1471,7 +1471,7 @@ class SchemaClass:
   def genMethodIdDeclarations (self, stream, variables):
     number = 1
     for method in self.methods:
-      stream.write ("    static const uint32_t METHOD_" + method.getName().upper() +\
+      stream.write ("    QPID_BROKER_EXTERN static const uint32_t METHOD_" + method.getName().upper() +\
                     " = %d;\n" % number)
       number = number + 1
 
@@ -1498,6 +1498,10 @@ class SchemaClass:
   def genNamePackageLower (self, stream, variables):
     stream.write (self.packageName.lower ())
 
+  def genPackageNameUpper (self, stream, variables):
+    up = "_".join(self.packageName.split("."))
+    stream.write (up.upper())
+
   def genNameUpper (self, stream, variables):
     stream.write (self.name.upper ())
 
@@ -1642,6 +1646,7 @@ class SchemaPackage:
   def genNamespace (self, stream, variables):
     stream.write("::".join(self.packageName.split(".")))
 
+
   def genOpenNamespaces (self, stream, variables):
     for item in self.packageName.split("."):
       stream.write ("namespace %s {\n" % item)
